package eu.ruekov.ruakij.LinkedBeaconTeleporters.linkedBeaconTeleporter;

import eu.ruekov.ruakij.LinkedBeaconTeleporters.Function;
import eu.ruekov.ruakij.LinkedBeaconTeleporters.Main;
import org.bukkit.Location;
import org.bukkit.Material;
import org.bukkit.block.Block;
import org.bukkit.configuration.ConfigurationSection;
import org.bukkit.inventory.ItemStack;
import org.bukkit.inventory.meta.ItemMeta;

import java.util.*;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class LinkedBeaconTeleporterManager {
    // Stores all placed Beacon-Teleporters for fast access
    // TODO: Evaluate necessity of 2 HashMaps with main reason of performance
    HashMap<String, List<LinkedBeaconTeleporterBlock>> linkedBeaconTeleporterById = new HashMap();
    HashMap<String, LinkedBeaconTeleporterBlock> linkedBeaconTeleporterByLoc = new HashMap();

    // Regex to match item-name&lore and extract id (group 1)
    String regex;

    public LinkedBeaconTeleporterManager(){
        Main.log.info("Loading LinkedBeaconTeleporter from config..");

        // Construct search item-regex
        regex = constructRegex(
                Main.config.getString("item.name"),
                Main.config.getStringList("item.lore")
        );

        // Load teleporters
        for(String teleporterId : Main.data.getKeys(false)){
            ConfigurationSection teleportersData = Main.data.getConfigurationSection(teleporterId);

            // New list for this id
            ArrayList<LinkedBeaconTeleporterBlock> lbtBlocks = new ArrayList();

            for(String blockId : teleportersData.getKeys(false)) {
                ConfigurationSection teleporterData = teleportersData.getConfigurationSection(blockId);

                // Load location
                String serializedLoc = teleporterData.getString("loc");
                Location loc;
                try {
                    loc = Function.deserializeBlockLocation(teleporterData.getString("loc"));
                } catch (InvalidPropertiesFormatException e) {
                    Main.log.severe("Could not load location='"+ serializedLoc +"' from "+ teleporterId +" > "+ blockId);
                    continue;
                }

                // Check if block is of type Beacon (in case the world was changed without the plugin running)
                Block block = loc.getBlock();
                if(block.getType() != Material.BEACON){
                    // Data out of sync! Skipping
                    Main.log.warning(teleporterId +" > "+ blockId +" at location "+ serializedLoc +" changed and is no longer LinkedBlockTeleporter! (Changed without the plugin running?)");
                    continue;
                }

                // Construct block
                LinkedBeaconTeleporterBlock lbtBlock = new LinkedBeaconTeleporterBlock(teleporterId, block);

                // Add block to id-list
                lbtBlocks.add(lbtBlock);

                // Save to loc-list
                linkedBeaconTeleporterByLoc.put(serializedLoc, lbtBlock);
            }

            // Save list to id-list
            linkedBeaconTeleporterById.put(teleporterId, lbtBlocks);
        }

        Main.log.info("All done!");
    }

    // Conversion-methods
    public ItemStack getItemStackFromLbtItem(LinkedBeaconTeleporterItem lbtItem){
        ItemStack item = new ItemStack(Material.BEACON);
        ItemMeta itemMeta = item.getItemMeta();

        // Get config-values
        String name = Main.config.getString("item.name");
        List<String> lore = Main.config.getStringList("item.lore");

        // Replace parameters
        name = name.replace("%id%", lbtItem.teleporterId);
        for(int i=0; i<lore.size(); i++){
            lore.set(i,
                    lore.get(i).replace("%id%", lbtItem.teleporterId)
            );
        }

        // Set meta
        itemMeta.setDisplayName(name);
        itemMeta.setLore(lore);

        item.setItemMeta(itemMeta);

        return item;
    }
    public LinkedBeaconTeleporterItem getLbtItemFromItemStack(ItemStack itemStack){
        ItemMeta itemMeta = itemStack.getItemMeta();

        // Serialize
        String nameAndLore = serialiseBeaconItem(
                itemMeta.getDisplayName(),
                itemMeta.getLore()
        );

        // Regex match
        Pattern pattern = Pattern.compile(regex);
        Matcher matcher = pattern.matcher(nameAndLore);
        if(!matcher.find())
            // No data
            return null;

        LinkedBeaconTeleporterItem lbtItem = new LinkedBeaconTeleporterItem(
                matcher.group(1),
                itemStack
        );

        return lbtItem;
    }

    public LinkedBeaconTeleporterBlock getLbtBlockFromLocation(Location loc){
        // Serialize
        String serializedLoc = Function.serialiseBlockLocation(loc);

        return linkedBeaconTeleporterByLoc.get(serializedLoc);
    }
    public List<LinkedBeaconTeleporterBlock> getLbtBlocksFromTeleportId(String teleportId){
        return linkedBeaconTeleporterById.get(teleportId);
    }

    String constructRegex(String name, List<String> lore){
        // Serialize
        String regex = serialiseBeaconItem(name, lore);

        // Wrap in regex-quotes
        regex = "^\\Q"+ regex +"\\E$";
        // Exclude id from regex-quotes and create group
        regex = regex.replace("%id%", "\\E(.*)\\Q");

        // Remove empty regex-quotes
        regex = regex.replace("\\E\\Q", "");

        return regex;
    }
    String serialiseBeaconItem(String name, List<String> lore){
        // get name and remove control-chars
        String str = name.replace("\\", "\\\\");
        if(lore != null){
            for (String loreLine : lore) {
                // get lore-lines and remove control-chars
                str += "\n"+ loreLine.replace("\\", "\\\\");
            }
        }

        return str;
    }
}
